#!/opt/rational/clearcase/bin/Perl -w
use strict;
#
# +--------------------------------------------------------------------------+
# | Copyright 2003 Rational Software Corporation.  All Rights Reserved.      |
# | This software contains proprietary and confidential information of       |
# | Rational and its suppliers.  Use, disclosure or reproduction is          |
# | prohibited without the prior express written consent of Rational.        |
# +--------------------------------------------------------------------------+
#
# This script reads any files provided (or stdin if there are no files), a line
# at a time, and does the specified processing.  The input is expected to
# consist of files (or lines) like mvfs_log files that contain lines like:
#
#   36d311f1 mvfs: Ok: INFO: View op replica_root delayed 109 seconds!
#   36d311f1 mvfs: Ok: INFO: replica root for VOB foo:/vs/bar took a long time!
#
# The regular expression matches lines that consist of hex digits, followed by
# whitespace and the rest of the line.  If the line matches, the hex digits are
# converted to decimal and fed to localtime() or gmtime(), which converts a time
# value to a date-time string when run in a scalar context (otherwise, they
# return a list of hour, minute, second, day, etc.).  The date-time string is
# then substituted for the original number in the line.  If a line doesn't
# match, it is left alone.
#
sub usage {
    my $prog_name = $_[0];

    print <<END_OF_USAGE;
PURPOSE:
    A filter to convert hexadecimal timestamps in a log file (e.g. mvfs_log) to
    readable dates.

USAGE:
    $prog_name < INLOGFILE > OUTPUT
    $prog_name INLOGFILE

OPTIONS:
    [-local] convert timestamps to local time (default).
    [-gmt]   convert timestamps to GMT (overrides local).
    [-help]  print this usage message.

EXAMPLES:
    # Pipe some log lines as input and write to stdout.
    head /var/adm/atria/log/mvfs_log | $prog_name

    # Read a log file and write the converted output to a file.
    $prog_name /var/adm/atria/log/mvfs_log > converted_log

    # Convert individual lines or timestamps until done.
    $prog_name
      <paste in a line from the log>
      <paste in a hexadecimal timestamp>
      ^D

NOTES:
    This command acts as a standard filter, so the logfile input can be piped or
    redirected to it, and the output can be piped or redirected, as appropriate.
END_OF_USAGE
    print "\n";
    print "Last modified: 20 Feb 2003";
    print "\n";
}

# Hack to make subroutine references work "right".  We don't seem to be able to
# use localtime() and gmtime() directly (e.g. \&gmtime) because we get undefined
# subroutine errors (even when trying to qualify them with the CORE package
# name).  The function reference only seems to work for locally defined
# functions (or "real" package functions).  We'll include the "scalar" context
# here, as well, for modularity.
#
sub my_gmtime {
    return scalar(CORE::gmtime($_[0]));
}

sub my_localtime {
    return scalar(CORE::localtime($_[0]));
}

# Process command arguments.
#
use File::Basename;
use Getopt::Long;

my $prog_name = basename($0);
my $opt_local;
my $opt_gmt;
my $opt_help;
my $converter;

GetOptions("local" => \$opt_local,
           "gmt" => \$opt_gmt,
           "help" => \$opt_help,
           );

# If --help, then show usage and we're done.
#
if (defined($opt_help)) {
    &usage($prog_name);
    exit;
}

# Local time is the default, and if both are specified, GMT wins.
#
if (defined($opt_gmt)) {
    # GMT always wins.
    $converter = \&my_gmtime;
} elsif (defined($opt_local)) {
    # localtime requested.
    $converter = \&my_localtime;
} else {
    # Default is localtime.
    $converter = \&my_localtime;
}
  
# Now process all the input files or stdin line by line with output to stdout.
#
while (<>) {
    s/^([a-fA-F0-9]+)(\s+.*)/sprintf("%s%s", &$converter(hex($1)), $2)/oe;
} continue {
    print;
}

# As an aside, you could use the following ksh function to convert a
# single time value to a date-time string (you need the "0x" in front
# of the time if it is hex):
#
#   ctime()
#   {
#      perl -e "print(scalar(localtime(${1})), \"\\n\")"
#   }
#
# As a further note, this script could be compressed to one line (if no arg
# processing is needed) by using the "-p" option to perl:
#
# #!/opt/rational/clearcase/bin/Perl -p
# s/^([a-fA-F0-9]+)(\s+.*)/sprintf("%s%s", scalar(localtime(hex($1))), $2)/oe;
